@@ -2,7 +2,7 @@
  *
  * Copyright 1999,2000,2001 BrightStation PLC
  * Copyright 2002 Ananova Ltd
- * Copyright 2002,2003,2004,2005,2006,2007,2008,2009,2010,2011 Olly Betts
+ * Copyright 2002,2003,2004,2005,2006,2007,2008,2009,2010,2011,2013 Olly Betts
  * Copyright 2008 Lemur Consulting Ltd
  * Copyright 2010 Richard Boulton
  *
@@ -202,7 +202,7 @@ ChertTable::read_block(uint4 n, byte * p) const
 	ssize_t bytes_read = pread(handle, reinterpret_cast<char *>(p), m,
 				   offset);
 	// normal case - read succeeded, so return.
-	if (bytes_read == m) return;
+	if (bytes_read == m) break;
 	if (bytes_read == -1) {
 	    if (errno == EINTR) continue;
 	    if (errno == EBADF && handle == -2)
@@ -233,6 +233,13 @@ ChertTable::read_block(uint4 n, byte * p) const
 
     io_read(handle, reinterpret_cast<char *>(p), block_size, block_size);
 #endif
+
+    int dir_end = DIR_END(p);
+    if (rare(dir_end < DIR_START || unsigned(dir_end) > block_size)) {
+	string msg("dir_end invalid in block ");
+	msg += str(n);
+	throw Xapian::DatabaseCorruptError(msg);
+    }
 }
 
 /** write_block(n, p) writes block n in the DB file from address p.
@@ -1681,8 +1688,8 @@ ChertTable::lazy_alloc_inflate_zstream() const {
 bool
 ChertTable::exists() const {
     LOGCALL(DB, bool, "ChertTable::exists", NO_ARGS);
-    return (file_exists(name + "DB") &&
-	    (file_exists(name + "baseA") || file_exists(name + "baseB")));
+    RETURN(file_exists(name + "DB") &&
+	   (file_exists(name + "baseA") || file_exists(name + "baseB")));
 }
 
 void
@@ -1718,7 +1725,6 @@ ChertTable::create_and_open(unsigned int block_size_)
     Assert(writable);
     close();
 
-    if (block_size_ == 0) abort();
     set_block_size(block_size_);
 
     // FIXME: it would be good to arrange that this works such that there's
@@ -1730,9 +1736,12 @@ ChertTable::create_and_open(unsigned int block_size_)
     /* create the base file */
     ChertTable_base base_;
     base_.set_revision(revision_number);
-    base_.set_block_size(block_size_);
+    base_.set_block_size(block_size);
     base_.set_have_fakeroot(true);
     base_.set_sequential(true);
+    // Doing a full sync here would be overly paranoid, as an empty table
+    // contains no precious data and xapian-check can recreate lost base
+    // files.
     base_.write_to_file(name + "baseA", 'A', string(), -1, NULL);
 
     /* remove the alternative base file, if any */
@@ -1871,7 +1880,7 @@ ChertTable::commit(chert_revision_number_t revision, int changes_fd,
 	// Do this as late as possible to allow maximum time for writes to
 	// happen, and so the calls to io_sync() are adjacent which may be
 	// more efficient, at least with some Linux kernel versions.
-	if (!io_sync(handle)) {
+	if (changes_tail ? !io_full_sync(handle) : !io_sync(handle)) {
 	    (void)::close(handle);
 	    handle = -1;
 	    (void)unlink(tmp);
