@@ -895,13 +895,13 @@ DEFINE_TESTCASE(blocksize1, brass || chert || flint) {
     string db_dir = "." + get_dbtype();
     mkdir(db_dir.c_str(), 0755);
     db_dir += "/db__blocksize1";
-    Xapian::WritableDatabase db;
     static const unsigned bad_sizes[] = {
 	65537, 8000, 2000, 1024, 16, 7, 3, 1, 0
     };
     for (size_t i = 0; i < sizeof(bad_sizes) / sizeof(bad_sizes[0]); ++i) {
 	size_t block_size = bad_sizes[i];
 	rm_rf(db_dir);
+	Xapian::WritableDatabase db;
 	if (get_dbtype() == "chert") {
 #ifdef XAPIAN_HAS_CHERT_BACKEND
 	    db = Xapian::Chert::open(db_dir, Xapian::DB_CREATE, block_size);
@@ -929,3 +929,32 @@ DEFINE_TESTCASE(blocksize1, brass || chert || flint) {
     }
     return true;
 }
+
+static void
+make_phrasebug1_db(Xapian::WritableDatabase &db, const string &)
+{
+    Xapian::Document doc;
+    doc.add_posting("hurricane", 199881);
+    doc.add_posting("hurricane", 203084);
+    doc.add_posting("katrina", 199882);
+    doc.add_posting("katrina", 202473);
+    doc.add_posting("katrina", 203085);
+    db.add_document(doc);
+}
+
+/// Regression test for ticket#653, fixed in 1.3.2 and 1.2.19.
+DEFINE_TESTCASE(phrasebug1, generated && positional) {
+    Xapian::Database db = get_database("phrasebug1", make_phrasebug1_db);
+    const char * qterms[] = { "katrina", "hurricane" };
+    Xapian::Enquire e(db);
+    Xapian::Query q(q.OP_PHRASE, qterms, qterms + 2, 5);
+    e.set_query(q);
+    Xapian::MSet mset = e.get_mset(0, 100);
+    TEST_EQUAL(mset.size(), 0);
+    const char * qterms2[] = { "hurricane", "katrina" };
+    Xapian::Query q2(q.OP_PHRASE, qterms2, qterms2 + 2, 5);
+    e.set_query(q2);
+    mset = e.get_mset(0, 100);
+    TEST_EQUAL(mset.size(), 1);
+    return true;
+}
